unit GLRenderToVideo;

interface

uses
  Windows,
  SysUtils,
  Graphics,
  MMSystem,
  VFW,
  DIBitmap;

resourcestring
  ERR_AVI_FILEOPEN = 'Could not open/create AVI file %s';
  ERR_AVI_CREATESTREAM = 'Could not create AVI stream';
  ERR_AVI_SETFORMAT = 'Could not set format';
  ERR_AVI_ADDFRAME = 'Could not add frame';

type
  EAVIFileOpenError = class(Exception);
  EAVICreateStreamError = class(Exception);
  EAVISetFormatError = class(Exception);
  EAVIAddFrameError = class(Exception);

  TAVIColorDepth=(cd_1_bit,
                  cd_4_bit,
                  cd_8_bit,
                  cd_16_bit,
                  cd_24_bit);

  TGLRenderToVideo = class
    private
    FFilename: string;
    FRate: cardinal;
    FAVIFile: PAVIFile;
    FAVIStreamInfo: TAVIStreamInfo;
    FpAVIStream: PAVIStream;
    FColorDepth: TAVIColorDepth;
    FInternalColorDepth: cardinal;
    FCurrFrame: cardinal;
    //Variables used internally
    bitmapInfo: PBitmapInfoHeader;
    bitmapBits: pointer;
    bitmapSize: cardinal;
    procedure SetColorDepth(const Value: TAVIColorDepth);
    protected
    public
    constructor Create;
    procedure BeginCapture(aBitmap: TBitmap);
    procedure AddFrame(aFrame:TBitmap);
    procedure EndCapture;
    property Filename: string read FFilename write FFilename;
    property Rate: cardinal read FRate write FRate;
    property ColorDepth: TAVIColorDepth read FColorDepth write SetColorDepth;
    property CurrFrame: cardinal read FCurrFrame;
  end;

implementation

{ TGLRenderToVideo }

procedure TGLRenderToVideo.AddFrame(aFrame: TBitmap);
var
  nul: LongInt;
begin
  with aFrame do
  begin
    InternalGetDIB(Handle, 0, bitmapInfo^, bitmapBits^, FInternalColorDepth);
    if AVIStreamWrite(FpAVIStream, FCurrFrame, 1, bitmapBits, bitmapSize,
                      AVIIF_KEYFRAME, nul, nul) <> AVIERR_OK then
    begin
      raise EAVIAddFrameError.Create(ERR_AVI_ADDFRAME);
      exit;
    end;
  end;
  Inc(FCurrFrame);
end;

procedure TGLRenderToVideo.BeginCapture(aBitmap: TBitmap);
var
  bitmapinfoSize: integer;
begin
  //Initialize the file
  AVIFileInit;

  if AVIFileOpen(FAVIfile, PChar(FFileName), OF_WRITE or OF_CREATE, nil) = AVIERR_OK then
  begin
    // Initialize the stream using data from the first bitmap
    FillChar(FAVIStreamInfo,SizeOf(FAVIStreamInfo),0);
    with FAVIStreamInfo do
    begin
      fccType := streamtypeVIDEO;
      fccHandler := 0;
      dwScale := 1;
      dwRate := FRate;
    end;

    with aBitmap do
    begin
      InternalGetDIBSizes(Handle, bitmapinfoSize, bitmapSize, FInternalColorDepth);
      bitmapInfo := AllocMem(bitmapinfoSize);
      bitmapBits := AllocMem(bitmapsize);
      InternalGetDIB(Handle, 0, bitmapInfo^, bitmapBits^, FInternalColorDepth);
    end;

    FAVIStreamInfo.dwSuggestedBufferSize := bitmapInfo^.biSizeImage;
    FAVIStreamInfo.rcFrame.Right := bitmapInfo^.biWidth;
    FAVIStreamInfo.rcFrame.Bottom:= bitmapInfo^.biHeight;

    if AVIFileCreateStream(FAVIFile, FpAVIStream, FAVIStreamInfo) = AVIERR_OK then
    begin
      with aBitmap do
      begin
        InternalGetDIB(Handle, 0, bitmapInfo^, bitmapBits^, FInternalColorDepth);
        if AVIStreamSetFormat(FpAVIStream, 0, bitmapInfo, bitmapInfoSize) = AVIERR_OK then
        begin
          // Write the 0th frame
          FCurrFrame := 0;
          AddFrame(aBitmap);
        end
        else
          raise EAVISetFormatError.Create(ERR_AVI_SETFORMAT);
      end;
    end
    else
      raise EAVICreateStreamError.Create(ERR_AVI_CREATESTREAM);
  end
  else
    raise EAVIFileOpenError.CreateFmt(ERR_AVI_FILEOPEN, [FFilename]);
end;

constructor TGLRenderToVideo.Create;
begin
  FFilename := '';
  SetColorDepth(cd_16_bit);
end;

procedure TGLRenderToVideo.EndCapture;
begin
  FreeMem(bitmapInfo);
  FreeMem(bitmapBits);

  AVIStreamRelease(FpAVIStream);
  AVIFileRelease(FAVIFile);
  AVIFileExit;
end;

procedure TGLRenderToVideo.SetColorDepth(const Value: TAVIColorDepth);
begin
  FColorDepth := Value;
  case FColorDepth of
    cd_1_bit:FInternalColorDepth := AVI_1_BIT_COLOR;
    cd_4_bit: FInternalColorDepth := AVI_4_BIT_COLOR;
    cd_8_bit: FInternalColorDepth := AVI_8_BIT_COLOR;
    cd_16_bit: FInternalColorDepth := AVI_16_BIT_COLOR;
    cd_24_bit: FInternalColorDepth := AVI_24_BIT_COLOR;
  end;
end;

end.
